' Updated for: AU 2019 - Managing CAD Standards with VB.NET - AS322865
' Originally created for: AU 2010 - CP433-4 - Managing CAD Standards with VB.NET
' Created by: Lee Ambrosius
'
' Based on ADN sample 'AcStVB7Sample' and work done by Kean Walmsley.
'
' AutoCAD
' Basic Layer CAD Standards Plug-in
'
' This plug-in does not support copying of a layer from the Standards file to
' the file being checked, nor does it support the ability to merge a layer that
' does not match a standard to one that does.
'
' This sample does support both the CAD Standards dialog in AutoCAD and the
' Batch Standards Checker.
'
' Basic flow of the plug-in for the Batch Standards Checker is:
'   - Initialize()
'   - GetObjectFilter()
'   - SetupForAudit()
'   - SetContext()
'   - Start()
'       - Next()
'       - Done()
'   - WritePluginInfo()
'   - CheckSysvar()
'   - Clear()
'
'
' Basic flow of the plug-in inside of AutoCAD
'
' At start up/opening a new drawing:
'   - Initialize()
'   - GetObjectFilter()
'   - SetupForAudit()
'
' Assign a standards plug-in:
'   - Clear()
'
' Check standards:
'   - Initialize()
'   - GetObjectFilter()
'   - SetupForAudit()
'   - SetContext()
'   - Start()
'       (Check Standards dialog box is displayed here)
'       - Next()
'       - Done()
'   - Clear()

Option Strict Off
Option Explicit On
Imports System.Runtime.InteropServices
Imports AXDBLib
Imports AULayersPlugin.StandardsHelpers

<ProgId("AUPlugins.BasicLayers"), ComClass()> _
Public Class BasicLayers
    Implements AcStMgr.IAcStPlugin2

    ' Global variables used for the plug-in
    Private m_pManager As AcStMgr.AcStManager
    Private m_pPlugin As BasicLayers
    Private m_pCheckDatabase As AcadDatabase
    Private m_pDWSDatabase As AcadDatabase
    Private m_pError As AcStMgr.AcStError
    Private m_FixArray As Object

    ' Custom objects defined from the Standards Helper class
    Private m_ContextList As New ObjectContextList()
    Private m_LayerCacheArray() As LayerCache

    Private m_RecommendedFixIndex As Integer
    Private m_curIndex As Integer
    Private m_FixCnt As Integer
    Private m_ex As Exception

    ' Property names
    Private m_sPropName As String
    Private m_sPropClrName As String
    Private m_sPropLWName As String

    ' XML document object used for reporting by the Batch Standards Checker
    Private m_xmlDoc As MSXML2.IXMLDOMDocument


    '' ****** Required for both CAD Standards and Batch Standards Checker features ******
    ' See "Developing and Implementing CAD Standards Plug-ins" in the CAD Standards Plug-ins Library Developer's Guide

    ' Defines the Author property for the plug-in, usually populated with a company name
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property Author() As String _
                    Implements AcStMgr.IAcStPlugin2.Author
        Get
            Return "Lee Ambrosius, Autodesk, Inc."
        End Get
    End Property

    ' Defines the Description property for the plug-in
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property Description() As String _
                    Implements AcStMgr.IAcStPlugin2.Description
        Get
            Return "Basic example of a Layer CAD Standards plug-in."
        End Get
    End Property

    ' Defines the HRef property for the plug-in; usually is assigned URL where the plug-in can be downloaded from.
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property HRef() As String _
                    Implements AcStMgr.IAcStPlugin2.HRef
        Get
            Return "http://www.hyperpics.com/cadplugins"
        End Get
    End Property

    ' Defines the Icon property for the plug-in
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property Icon() As Long _
                    Implements AcStMgr.IAcStPlugin2.Icon
        Get
            Return 1
        End Get
    End Property

    ' Defines the Name property for the plug-in; the name that appears in the 
    ' CAD Standards dialog box and the Batch Standards Checker.
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property Name() As String _
                    Implements AcStMgr.IAcStPlugin2.Name
        Get
            Return "Basic Layers (AU 2019)"
        End Get
    End Property

    ' Defines the Version property for the plug-in
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public ReadOnly Property Version() As String _
                    Implements AcStMgr.IAcStPlugin2.Version
        Get
            Return "1.0"
        End Get
    End Property

    ' Defines an array of the type of objects (class names) that this plug-in can check
    ' It is recommened to use only one type of object for each plug-in
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public Function GetObjectFilter() As Object _
           Implements AcStMgr.IAcStPlugin2.GetObjectFilter

        ' Array of objects
        Dim sFilterArray(0) As String

        ' *** Change the following to support a different object type
        sFilterArray(0) = "AcDbLayerTableRecord"
        Return sFilterArray
    End Function

    '' ****** Required for both the CAD Standards and Batch Standards Checker features ******
    ' See the "Implement Initialization Methods" section of "Developing and Implementing CAD Standards Plug-ins"
    ' in the CAD Standards Plug-ins Library Developer's Guide

    ' Initializes the plugin
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public Sub Initialize(ByVal pStdsMgr As AcStMgr.AcStManager) _
               Implements AcStMgr.IAcStPlugin2.Initialize

        ' Store pointers to the Standards Manager object and plug-in in global variables
        m_pManager = pStdsMgr
        m_pPlugin = Me
    End Sub

    ' Sets the objects to validate when checking for errors
    ' Required: Yes (Used by CAD Standards/Batch Standards Checker)
    Public Sub SetupForAudit(ByVal pDb As AcadDatabase, _
                             ByVal szPathName As String, _
                             ByVal stdNameArray As Object, _
                             ByVal stdPathArray As Object, _
                             ByVal stdDbArray As Object) _
                                   Implements AcStMgr.IAcStPlugin2.SetupForAudit

        ' Used to define the context in which the plug-in will be used.

        ' The drawing to be checked and the DWS files being used 
        ' are received at this point. Store the DWS standards definitions
        ' and the objects to be checked. See GetObjectFilter() for the
        ' types of objects the plug-in should check.

        ' WARNING!: Do not save a reference to the database object
        ' as it may not be valid going forward.

        If Not IsNothing(pDb) Then
            '' *** Change the following to support a different object type
            Dim pStdLayer As AcadLayer
            Dim i, iDWS As Integer

            ' Store a reference to the DWG to be checked in a global variable
            m_pCheckDatabase = pDb

            ' Verify the reference to the DWG to be checked is valid
            If Not IsNothing(m_pCheckDatabase) Then
                ' *** Change the following to support a different object type
                For Each acObj As AcadObject In m_pCheckDatabase.Layers
                    If acObj.ObjectName = "AcDbLayerTableRecord" Then
                        m_ContextList.Add(acObj.ObjectID, True)
                    End If
                Next
            End If

            i = 0
            ' Iterate each of the DWS files and cache the properties to be checked
            For iDWS = 0 To UBound(stdDbArray)
                ' Get the DWS databases to be used when checking for errors
                m_pDWSDatabase = stdDbArray(iDWS)

                ' *** Change the following to support a different object type
                For Each pStdLayer In m_pDWSDatabase.Layers
                    ' Cache the properties for the objects in the DWS files
                    Dim layerCache As New LayerCache()
                    layerCache.Name = pStdLayer.Name
                    layerCache.Color = pStdLayer.color
                    layerCache.Lineweight = pStdLayer.Lineweight
                    layerCache.StandardFileName = stdNameArray(iDWS)

                    ' Save the fix information to be used later to correct the standards violation
                    Dim pFix As New AcStMgr.AcStFix()
                    pFix.Description = "Layer fix"
                    pFix.StandardFileName = layerCache.StandardFileName
                    pFix.FixObjectName = pStdLayer.Name

                    If pFix.PropertyCount = 0 Then
                        pFix.PropertyValuePut("Color", pStdLayer.color)
                        pFix.PropertyValuePut("Lineweight", pStdLayer.Lineweight)
                    End If

                    layerCache.StandardsFix = pFix

                    ReDim Preserve m_LayerCacheArray(i)
                    m_LayerCacheArray(i) = layerCache
                    layerCache = Nothing
                    pFix = Nothing
                    i = i + 1
                Next

                m_pDWSDatabase = Nothing
            Next
        End If
    End Sub


    '' ****** Required for CAD Standards to support runtime checking ******
    '' ****** Not required for Batch Standards Checker ******
    ' See the "Implement Error Iteration Methods" section of "Developing and Implementing CAD Standards Plug-ins"
    ' in the CAD Standards Plug-ins Library Developer's Guide

    ' Sets the context that the plug-in should be used to check a drawing
    ' Required: No, unless runtime checking is required (Used by CAD Standards)
    Public Sub SetContext(ByVal objIdArray As Object, ByVal bUseDb As Boolean) _
                          Implements AcStMgr.IAcStPlugin2.SetContext

        ' If useDb is set to "true" (default), or if
        ' objIdArray is blank, we use the database (we get
        ' all ids for the current drawing). Otherwise, we
        ' pass along the supplied list of objects.
        m_ContextList.SetContext(bUseDb, objIdArray)
    End Sub

    ' Initializes the checking mechanism
    ' Required: No, unless runtime checking is required (Used by CAD Standards)
    Public Sub PlugIn_Start(ByVal pStartError As AcStMgr.AcStError) _
                            Implements AcStMgr.IAcStPlugin2.Start

        ' pErr is not an actual programming error that needs to be handled,
        ' but a reference to an object that does not match the standards
        ' in the DWS file.

        ' If pErr is an object, checking should start
        ' from that error, not the beginning of the current context.
        If IsNothing(pStartError) = False Then

            ' Get the ObjectId of the first object to check against
            Dim badId As Long = pStartError.BadObjectId

            ' Find the index for BadObjectId in m_objIDArray
            For m_curIndex = 0 To m_ContextList.Count - 1
                If m_ContextList.Item(m_curIndex) = badId Then
                    m_curIndex = m_curIndex - 1
                    PlugIn_Next()
                End If
            Next
        Else
            ' A valid AcStError object was not passed in. Start checking
            ' at the begining of the array of objects.
            m_curIndex = -1
            PlugIn_Next()
        End If
    End Sub

    ' Go to the next error in the current context
    ' Required: No, unless runtime checking is required (Used by CAD Standards)
    Public Sub PlugIn_Next() _
               Implements AcStMgr.IAcStPlugin2.Next
        m_pError = Nothing

        If m_ContextList.Count > 0 Then

            ' *** Change the following to support a different object type
            ' Drawing contains AcDbLayer objects
            Dim layerObj As AcadLayer
            Dim iCache As Integer
            Dim bFoundError As Boolean, bNameFoundError As Boolean
            If m_LayerCacheArray.Length() > 0 Then

                ' Make sure we have not reached the last object to be checked 
                If m_curIndex < m_ContextList.Count - 1 Then
                    m_curIndex = m_curIndex + 1
                    bFoundError = False
                    bNameFoundError = False

                    ' Iterate the objects that need to be checked
                    While m_curIndex < m_ContextList.Count

                        ' Get the ObjectId of the object we are checking
                        layerObj = m_pCheckDatabase.ObjectIdToObject(m_ContextList.Item(m_curIndex))

                        ' Iterate through each of the standard items
                        For iCache = LBound(m_LayerCacheArray) To UBound(m_LayerCacheArray)

                            ' Compare the names of the objects, if they don't match 
                            ' then flag the object as a possible error
                            If (layerObj.Name.CompareTo(m_LayerCacheArray(iCache).Name) <> 0) Then
                                bFoundError = True
                                bNameFoundError = True
                            Else
                                ' Names matched
                                ' Now compare colors and lightweights for matches,
                                ' if not flag the object as a possible error
                                If layerObj.color.ToString() <> m_LayerCacheArray(iCache).Color.ToString() Or
                                   layerObj.Lineweight.ToString() <> m_LayerCacheArray(iCache).Lineweight.ToString() Then

                                    ' Color and lineweights did not match
                                    bFoundError = True
                                    bNameFoundError = False
                                Else
                                    ' Colors and lineweights matched, do not flag as error and exit the For statement
                                    bFoundError = False
                                    bNameFoundError = False
                                End If

                                Exit For
                            End If
                        Next

                        ' Let the user know about the errors we found for the object
                        If bFoundError = True Then
                            ' We found an error, so create a local error object
                            Dim pError As New AcStMgr.AcStError()

                            ' Description of error
                            pError.Description = "Layer is non-standard"
                            pError.BadObjectId = layerObj.ObjectID

                            ' Name of the error in the Batch Standards Checker
                            pError.BadObjectName = layerObj.Name
                            pError.Plugin = m_pPlugin
                            pError.ErrorTypeName = "Layer "
                            pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFlagsNone

                            ' Properties that are wrong
                            If pError.PropertyCount = 0 Then
                                pError.PropertyValuePut("Color", layerObj.color)
                                pError.PropertyValuePut("Lineweight", layerObj.Lineweight)
                            End If

                            m_pError = pError
                            bFoundError = False
                            bNameFoundError = False

                            Exit While
                        End If

                        m_curIndex = m_curIndex + 1
                    End While
                End If
            End If
        End If
    End Sub

    ' Returns True if all errors have been fixed or ignored
    ' Required: No, unless runtime checking is required (Used by CAD Standards)
    Public Function PlugIn_Done() As Boolean Implements AcStMgr.IAcStPlugin2.Done
        Return IsNothing(m_pError)
    End Function

    ' Cleans up after the plug-in; called before the plug-in is released from memory
    ' Required: No, unless runtime checking is required (Used by CAD Standards)
    Public Sub PlugIn_Clear() Implements AcStMgr.IAcStPlugin2.Clear

        ' If the XML Document object, the plug-in must be running
        ' in the Batch Standards Checker. Output the standards items that
        ' were used for the comparisons to the Standards Items section.
        If IsNothing(m_xmlDoc) = False Then
            WriteStandardsItemsInfo()
        End If

        ' Clean up the global variables
        m_xmlDoc = Nothing
        m_pPlugin = Nothing
        m_curIndex = -1
        m_RecommendedFixIndex = -1
        m_FixCnt = 0
        m_sPropClrName = ""
        m_sPropLWName = ""
        m_pManager = Nothing
        m_pDWSDatabase = Nothing
        m_pCheckDatabase = Nothing
        m_ex = Nothing

        If IsNothing(m_pError) = False Then
            m_pError.Reset()
            m_pError = Nothing
        End If

        ' *** Change the following to support a different object type
        If IsNothing(m_LayerCacheArray) = False Then
            Dim i As Integer
            For i = 0 To UBound(m_LayerCacheArray)
                If IsDBNull(m_LayerCacheArray(i).StandardsFix) = False Then
                    m_LayerCacheArray(i).StandardsFix.Reset()
                    m_LayerCacheArray(i).StandardsFix = Nothing
                End If
            Next

            Erase m_LayerCacheArray
        End If

        ' Remove the array from memory if it isn't empty
        If IsNothing(m_FixArray) = False Then
            Erase m_FixArray
        End If

        m_ContextList.Clear()
    End Sub


    '' ****** Used for CAD Standards and/or Batch Standards Checker features ******
    ' See the "Implement Error and Fix Retrieval Methods" section of "Developing and Implementing CAD Standards Plug-ins"
    ' in the CAD Standards Plug-ins Library Developer's Guide

    ' Returns the current error
    ' Required: Yes  (Used by CAD Standards/Batch Standards Checker)
    Public Function GetError() As AcStMgr.AcStError Implements AcStMgr.IAcStPlugin2.GetError
        Return m_pError
    End Function

    ' Returns an array of all the possible fixes for the given error (IAcStFix objects)
    ' Note: Objects in the array must be released
    ' Required: Yes (Used by CAD Standards)
    Public Sub GetAllFixes(ByVal pError As AcStMgr.AcStError, _
                           ByRef fixArray As Object, _
                           ByRef recommendedFixIndex As Integer) _
                           Implements AcStMgr.IAcStPlugin2.GetAllFixes

        ' *** Change the following to support a different object type
        If IsNothing(pError) = False Then
            Dim arr(UBound(m_LayerCacheArray)) As AcStMgr.IAcStFix
            Dim vErrorClrVal As ACAD_COLOR
            Dim vErrorLWVal As ACAD_LWEIGHT

            Dim i As Integer
            recommendedFixIndex = -1
            m_FixCnt = 0

            ' If we have a cache of fixes, then use that
            If m_LayerCacheArray.Length > 0 Then
                For i = LBound(m_LayerCacheArray) To UBound(m_LayerCacheArray)
                    vErrorClrVal = pError.PropertyValueGet("Color")
                    vErrorLWVal = pError.PropertyValueGet("Lineweight")
                    arr(i) = m_LayerCacheArray(i).StandardsFix
                Next

                fixArray = arr
                m_FixArray = fixArray
                Dim tmpFix As New AcStMgr.AcStFix()

                ' Find the recommendedFixIndex
                ' Function retrieves the index, not the fix object
                tmpFix = GetRecommendedFix(pError)
                tmpFix = Nothing
                recommendedFixIndex = m_RecommendedFixIndex
            End If

            ' If no recommended fix was found, then set the proper flag on the error object
            If recommendedFixIndex = -1 Then
                pError.ResultStatus = AcStMgr.AcStResultStatus.acStResNoRecommendedFix
            End If
        End If
    End Sub

    ' Gets the recommended fix object based on the Standards Error object
    ' Required: No, Optional  (Used by CAD Standards)
    Public Function GetRecommendedFix(ByVal pError As AcStMgr.AcStError) As AcStMgr.AcStFix _
                                      Implements AcStMgr.IAcStPlugin2.GetRecommendedFix

        Dim pRecommendedFix As New AcStMgr.AcStFix()

        ' *** Change the following to support a different object type
        Dim nameToBeChecked As String
        If m_LayerCacheArray.Length = 0 Then
            pError.ResultStatus = AcStMgr.AcStResultStatus.acStResNoRecommendedFix
        Else
            Dim tmpLayer As AcadLayer
            Dim layCache As LayerCache
            Dim tmpObjID As Long
            Dim i As Integer

            ' Get the objectId of the error
            tmpObjID = pError.BadObjectId()

            tmpLayer = m_pCheckDatabase.ObjectIdToObject(tmpObjID)
            nameToBeChecked = tmpLayer.Name
            tmpLayer = Nothing
            layCache = m_LayerCacheArray(0)

            m_RecommendedFixIndex = 0

            ' Attempt to get recommended fix object for the error object
            ' Rule: Layer names must be the same
            For i = 0 To UBound(m_LayerCacheArray)
                If m_LayerCacheArray(i).Name = nameToBeChecked Then
                    layCache = m_LayerCacheArray(i)
                    m_RecommendedFixIndex = i
                End If
            Next

            ' Populate the properties for the recommended fix object
            pRecommendedFix.Description = "Layer fix"
            pRecommendedFix.StandardFileName = m_LayerCacheArray( _
                                               m_RecommendedFixIndex).StandardFileName
            pRecommendedFix.FixObjectName = m_LayerCacheArray( _
                                               m_RecommendedFixIndex).Name

            If pRecommendedFix.PropertyCount = 0 Then
                pRecommendedFix.PropertyValuePut("Color", m_LayerCacheArray( _
                                                 m_RecommendedFixIndex).Color)
                pRecommendedFix.PropertyValuePut("Lineweight", m_LayerCacheArray( _
                                                 m_RecommendedFixIndex).Lineweight)
            End If
        End If

        GetRecommendedFix = pRecommendedFix
    End Function

    ' Provides information back in the provided arrays
    ' as to which error objects were fixed.
    ' Information contains the 'property name, current value, fix value'
    ' Required: Yes (Used by CAD Standards)
    Public Sub GetPropertyDiffs(ByVal pError As AcStMgr.AcStError, _
                                ByVal pFix As AcStMgr.AcStFix, _
                                ByRef pPropNames As Object, _
                                ByRef pErrorValues As Object, _
                                ByRef pFixValues As Object, _
                                ByRef pFixableStatuses As Object) _
                                Implements AcStMgr.IAcStPlugin2.GetPropertyDiffs

        If IsNothing(pError) = False And IsNothing(pFix) = False Then
            ' Define the array sizes based on the largest possible range, do not ReDim
            Dim sPropNames(1) As String
            Dim sErrorValues(1) As String
            Dim sFixValues(1) As String
            Dim bFixableStatuses(1) As Boolean

            Dim sPropName As String = ""
            Dim vErrorVal As Object = Nothing
            Dim vFixVal As Object = Nothing
            Dim i As Integer

            ' Iterate the properties of the error
            For i = 0 To pError.PropertyCount - 1
                ' Get the Lineweight property value
                pError.PropertyGetAt(i, sPropName, vErrorVal)
                m_sPropName = sPropName

                ' Retrieve the corresponding Fix property value
                Try
                    pFix.PropertyValueGet(sPropName, vFixVal)

                    ' Check to see if the fix and error values match, if not get the expected value
                    If (vErrorVal.CompareTo(vFixVal) <> 0) Then

                        ' Assign the property name and error/fix values
                        sPropNames(i) = sPropName
                        sErrorValues(i) = vErrorVal.ToString
                        sFixValues(i) = vFixVal.ToString
                        bFixableStatuses(i) = True
                    End If
                Catch
                    ' Catch and handle the error as needed
                End Try
            Next

            ' Return the arrays
            pPropNames = sPropNames
            pErrorValues = sErrorValues
            pFixValues = sFixValues
            pFixableStatuses = bFixableStatuses

            ' Clear the local arrays
            Erase sPropNames
            Erase sErrorValues
            Erase sFixValues
            Erase bFixableStatuses

            ' Increment the number of fixes
            m_FixCnt = m_FixCnt + 1
        End If
    End Sub

    '' ****** Used for CAD Standards only feature ******
    ' See the "Implement Error Fixing" section of "Developing and Implementing CAD Standards Plug-ins"
    ' in the CAD Standards Plug-ins Library Developer's Guide

    ' Attempts to use a fix object to change an error object
    ' Required: Yes  (Used by CAD Standards)
    Public Sub FixError(ByVal pError As AcStMgr.AcStError, _
                        ByVal pFix As AcStMgr.AcStFix, _
                        Optional ByRef pFailedReason As String = "0") _
                        Implements AcStMgr.IAcStPlugin2.FixError

        ' *** Change the following to support a different object type
        If IsNothing(pError) = False Then
            Dim badObj As AcadLayer
            Dim sFixClrVal As ACAD_COLOR
            Dim sFixLWVal As ACAD_COLOR
            Dim badObjID As Long

            badObjID = pError.BadObjectId

            ' Get the drawing object to fix
            badObj = m_pCheckDatabase.ObjectIdToObject(badObjID)

            ' If no fix is provided, then try the recommended fix
            If IsNothing(pFix) Then
                Dim tmpFix As New AcStMgr.AcStFix()
                tmpFix = GetRecommendedFix(pError)

                ' If not recommend fix is available, set the error's result status to noRecommendedFix
                If IsNothing(tmpFix) Then
                    pError.ResultStatus = AcStMgr.AcStResultStatus.acStResNoRecommendedFix
                Else
                    ' Get the object to fix
                    pFix = tmpFix
                    tmpFix = Nothing

                    ' Get and fix the color of the layer
                    pFix.PropertyValueGet("Color", sFixClrVal)

                    Try
                        badObj.color = sFixClrVal
                        pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixed
                    Catch m_ex As Exception
                        pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixFailed
                    End Try

                    ' Get and fix the Lineweight of the layer
                    pFix.PropertyValueGet("Lineweight", sFixLWVal)

                    Try
                        badObj.Lineweight = sFixLWVal
                        pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixed
                    Catch m_ex As Exception
                        pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixFailed
                    End Try
                End If
            Else
                ' Get and fix the color of the layer
                pFix.PropertyValueGet("Color", sFixClrVal)
                Try
                    badObj.color = sFixClrVal
                    pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixed
                Catch m_ex As Exception
                    pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixFailed
                End Try

                ' Get and fix the Lineweight of the layer
                pFix.PropertyValueGet("Lineweight", sFixLWVal)
                Try
                    badObj.Lineweight = sFixLWVal
                    pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixed
                Catch m_ex As Exception
                    pError.ResultStatus = AcStMgr.AcStResultStatus.acStResFixFailed
                End Try
            End If
        End If
    End Sub

    ' Updates the status of the Standard Error object
    Public Sub UpdateStatus(ByVal pError As AcStMgr.AcStError) _
                            Implements AcStMgr.IAcStPlugin2.UpdateStatus

    End Sub

    '' ****** Used for the Batch Standards Checker only ******
    ' See the "Implement Report Methods" section of "Developing and Implementing CAD Standards Plug-ins"
    ' in the CAD Standards Plug-ins Library Developer's Guide

    ' Used by the reporting feature of the Batch Standards Checker
    ' to get information about the plugin.
    ' Required: No, Optional (Used by Batch Standards Checker)
    Public Sub WritePluginInfo(ByVal pPluginInfoSectionNode As Object) _
                               Implements AcStMgr.IAcStPlugin2.WritePluginInfo

        ' Sets the Section node for the XML file output
        Dim pSectionNode As MSXML2.IXMLDOMNode = pPluginInfoSectionNode

        ' Creates a new element to write to
        Dim xmlElem As MSXML2.IXMLDOMElement = _
            pSectionNode.ownerDocument.createElement("AcStPluginInfo")

        ' Appends the new element to the XML file
        Dim pPluginInfoElement As MSXML2.IXMLDOMElement = _
            pSectionNode.appendChild(xmlElem)

        ' Writes the plug-in information to the XML file
        pPluginInfoElement.setAttribute("PluginName", Name())
        pPluginInfoElement.setAttribute("Version", Version())
        pPluginInfoElement.setAttribute("Description", Description())
        pPluginInfoElement.setAttribute("Author", Author())
        pPluginInfoElement.setAttribute("HRef", HRef())
        pPluginInfoElement.setAttribute("DWSName", "")
        pPluginInfoElement.setAttribute("Status", "1")

        ' Workaround for Standard Items list
        ' Save the XML document to the global variable so the Standards items list can be updated
        ' as there is no direct function to do this. Must be done in the Clear method since
        ' it is called last.
        m_xmlDoc = pSectionNode.ownerDocument
    End Sub

    ' Used to output information about the Standards items
    ' Workaround for what appears to be a gap in the API; called from Plugin-Clear()
    ' Required: No, Optional (Used by Batch Standards Checker)
    Public Sub WriteStandardsItemsInfo()

        If IsNothing(m_xmlDoc) = False Then
            Dim nCnt As Integer = 0

            ' Get the first instance of AcStDWSSection; there should only be one
            Dim pAcStDWSSection As MSXML2.IXMLDOMNode =
                                   m_xmlDoc.getElementsByTagName("AcStDWSSection")(0)

            If IsNothing(pAcStDWSSection) = False Then
                For Each child As MSXML2.IXMLDOMNode In pAcStDWSSection.childNodes
                    If child.nodeName = "AcStDWSFile" Then

                        For Each grandchild As MSXML2.IXMLDOMNode In child.childNodes
                            If grandchild.nodeName = "AcStDWSPlugin" Then

                                Dim pDWSId As String =
                                   grandchild.attributes.getNamedItem("DWSId").nodeValue.ToString()

                                Dim pPluginId As String =
                                   grandchild.attributes.getNamedItem("PluginId").nodeValue.ToString()

                                Dim pPluginName As String =
                                   grandchild.attributes.getNamedItem("PluginName").nodeValue.ToString()

                                Dim pPluginVersion As String =
                                   grandchild.attributes.getNamedItem("PluginVersion").nodeValue.ToString()

                                If Name() = pPluginName And Version() = pPluginVersion Then
                                    For Each pFix As LayerCache In m_LayerCacheArray

                                        Dim xmlElem As MSXML2.IXMLDOMElement =
                                            grandchild.ownerDocument.createElement("AcStDWSItem")

                                        ' Appends a new element to the XML file
                                        Dim pAcStDWSItemElement As MSXML2.IXMLDOMElement =
                                            grandchild.appendChild(xmlElem)

                                        ' Writes the standard item information to the XML file
                                        pAcStDWSItemElement.setAttribute("DWSId", pDWSId)
                                        pAcStDWSItemElement.setAttribute("PluginId", pPluginId)
                                        pAcStDWSItemElement.setAttribute("ItemName", pFix.Name)

                                        ' Add color
                                        Dim xmlElemProp1 As MSXML2.IXMLDOMElement =
                                            pAcStDWSItemElement.ownerDocument.createElement("Property")

                                        ' Appends a new element to the XML file
                                        Dim pAcStDWSItemElementProp1 As MSXML2.IXMLDOMElement =
                                            pAcStDWSItemElement.appendChild(xmlElemProp1)

                                        pAcStDWSItemElementProp1.setAttribute("ItemDescription", "Color = " + pFix.Color.ToString())

                                        ' Add lineweight
                                        Dim xmlElemProp2 As MSXML2.IXMLDOMElement =
                                            pAcStDWSItemElement.ownerDocument.createElement("Property")

                                        ' Appends a new element to the XML file
                                        Dim pAcStDWSItemElementProp2 As MSXML2.IXMLDOMElement =
                                            pAcStDWSItemElement.appendChild(xmlElemProp2)

                                        pAcStDWSItemElementProp2.setAttribute("ItemDescription", "Lineweight = " + pFix.Lineweight.ToString())
                                    Next
                                End If
                            End If
                        Next
                    End If
                Next
            End If
        End If
    End Sub

    '' ****** Unknown functionality; required by the CAD Standards plug-in ******
    ' Checks a system variable
    Public Sub CheckSysvar(ByVal syvarName As String, _
                           ByVal bGetAllFixes As Boolean, _
                           ByRef bPassFail As Boolean) _
                           Implements AcStMgr.IAcStPlugin2.CheckSysvar

        ' Not implemented in CAD Standards plug-in
    End Sub

    '' ****** Unknown functionality; required by the CAD Standards plug-in ******
    ' Specifies if the plugin uses information from the database for checking
    Public Sub StampDatabase(ByVal pDb As AcadDatabase, _
                             ByRef pStampIt As Boolean) _
                             Implements AcStMgr.IAcStPlugin2.StampDatabase

        ' If the DWS contains layers, return stampIt as true, 
        ' otherwise, return stampIt as false
        pStampIt = False
        If pDb.Layers.Count > 0 Then
            pStampIt = True
        End If
    End Sub
End Class
