﻿Imports AXDBLib

' Helper classes used for managing objects that should be checked
Public Class StandardsHelpers

#Region "Layer Cache Class"
    ' Store the "standard" layer properties to check for (Name, Color, Lineweight)
    ' from the Drawing Standards (DWS) files, as well as pointers to 
    ' the AcStFix object associated with the object 
    Public Class LayerCache
        Public Name As String
        Public Color As ACAD_COLOR
        Public Lineweight As ACAD_LWEIGHT
        Public StandardFileName As String
        Public StandardsFix As AcStMgr.AcStFix
    End Class

#End Region

#Region "Context List Class "
    ' Manages a list of all objects to be checked
    ' All in database, or just those recently added or modified (realtime)
    Public Class ObjectContextList

#Region "IdCollection Implementation"

        ' ObjectId collection class
        ' Used to create a list of all unique ObjectIds to be checked or that represent
        ' the standards to be checked against
        Private Class ObjectIdCollection
            Inherits System.Collections.CollectionBase

            ' Current index being worked with for Find and Add methods
            Private m_Index As Integer

            ' Adds an objectId to the list if it does not exist already
            Public Sub Add(ByVal pObjId As Long)
                If Find(pObjId) = -1 Then
                    Insert(m_Index, pObjId)
                End If
            End Sub

            ' Removes an ObjectId from the list
            Public Sub Remove(ByVal pObjId As Long)
                If Find(pObjId) >= 0 Then
                    List.RemoveAt(m_Index)
                End If
            End Sub

            ' Returns the the ObjectId of the item in the list based on an index value
            Public ReadOnly Property Item(ByVal pIndex As Integer) As Long
                Get
                    Return CType(List.Item(pIndex), Long)
                End Get
            End Property

            ' Inserts a new ObjectId in the list at a known location
            Public Sub Insert(ByVal pIndex As Integer, ByVal pObjId As Long)
                List.Insert(pIndex, pObjId)
            End Sub

            ' Locates the index of the ObjectId in the list
            ' The idex of the object is returned along with setting
            ' the value to the global m_Index variable
            Public Function Find(ByVal pObjId As Long) As Integer

                ' Search algorithm
                Dim nLow, nHigh, nIndex As Integer
                Dim bFound As Boolean
                nLow = 0
                nHigh = List.Count - 1
                While (bFound = False And nLow <= nHigh)
                    nIndex = (nLow + nHigh) / 2
                    If List.Item(nIndex) = pObjId Then
                        bFound = True
                    ElseIf List.Item(nIndex) < pObjId Then
                        nLow = nIndex + 1
                    Else
                        nHigh = nIndex - 1
                    End If
                End While

                ' Return the index for the item isf it was located in the list
                If bFound = True Then
                    m_Index = nIndex + 1
                    Return nIndex
                Else
                    m_Index = nLow
                    Return -1
                End If
            End Function
        End Class

#End Region

        ' List of objects to use for realtime checking
        Private m_RtChkIdArray As New ObjectIdCollection()

        ' List of all objects in the drawing to be checked
        Private m_DbIdArray As New ObjectIdCollection()

        ' Flag that determines which context the plug-in is operating in
        ' True - Drawing
        ' False - Realtime checking
        Private m_bUseDb As Boolean

        ' Set the context of objects to be checked
        Public Sub SetContext(ByVal pbUseDb As Boolean, ByVal pContextArray As Object)
            If pbUseDb = False And IsNothing(pContextArray) = False Then
                m_bUseDb = False

                ' Update the object id arrays for both realtime and the database
                For i As Integer = LBound(pContextArray) To UBound(pContextArray)
                    m_RtChkIdArray.Add(pContextArray(i))
                    m_DbIdArray.Add(pContextArray(i))
                Next
            Else
                m_bUseDb = True

                ' Clear the list of objects for realtime checking
                m_RtChkIdArray.Clear()
            End If
        End Sub

        ' Return the item from the array according to the bUseDb flag
        Public ReadOnly Property Item(ByVal index As Integer) As Object
            Get
                If m_bUseDb = True Then
                    Return m_DbIdArray.Item(index)
                Else
                    Return m_RtChkIdArray.Item(index)
                End If
            End Get
        End Property

        ' Add an item to the array according to the bUseDb flag
        Public Sub Add(ByVal pObjId As Long, ByVal bUseDb As Boolean)
            If bUseDb = True Then
                m_DbIdArray.Add(pObjId)
            Else
                m_RtChkIdArray.Add(pObjId)
            End If
        End Sub

        ' Clear both ObjectId arrays
        Public Sub Clear()
            m_RtChkIdArray.Clear()
            m_DbIdArray.Clear()
        End Sub

        ' Return the number of objects in the ObjectId array according to the bUseDb flag
        Public ReadOnly Property Count() As Integer
            Get
                If m_bUseDb = True Then
                    Return m_DbIdArray.Count
                Else
                    Return m_RtChkIdArray.Count
                End If
            End Get
        End Property
    End Class

#End Region
End Class
